import struct
from cStringIO import StringIO
from binascii import hexlify

class IterStruct(object):
    
    def __init__(self,f,unpack,bufsize = 64 * 1024):
        super(IterStruct,self).__init__()
        self.f = f
        self.unpack = unpack
        self.bufsize = bufsize
        self.buffer = None
    
    def __iter__(self):
        while True:
            yield self.next()
    
    def _read_chunk(self,leftover = ''):
        s = self.f.read(self.bufsize)
        if not s: 
            raise StopIteration
        self.buffer = StringIO(leftover + s)
    
    def _unpack(self):
        return self.unpack.unpack(self.buffer)
    
    def next(self):
        if not self.buffer:
            self._read_chunk()
        
        try:
            return self._unpack()
        except Exception:
            self._read_chunk(self.buffer.read())
        
        try:    
            return self._unpack()
        except Exception:
            # there was something left in the file, but it is not long
            # enough, or malformed.
            raise StopIteration

class IdentityEncoder(object):
    
    def __init__(self):
        super(IdentityEncoder,self).__init__()
    
    @classmethod
    def encode(cls,inst):
        return inst
    
    @classmethod
    def decode(cls,s):
        return s

class BaseStruct(object):
    
    _variable = ('_id',)
    _fieldnames = ()
    _struct = None
    _byte = struct.Struct('B')
    
    def __init__(self, bytestring = None,**kwargs):
        super(BaseStruct,self).__init__()
        
        self._bytestring = None
        
        if bytestring:
            self._bytestring = bytestring
            d = self.__class__._unpack(StringIO(bytestring))
            self.__dict__.update(d)
            return
        
        self.__dict__.update(kwargs)
    
    @classmethod
    def encode(cls,inst):
        return inst.pack()
    
    @classmethod
    def decode(cls,flo):
        return cls.unpack(flo)
    
    @classmethod
    def _unpack(cls,flo):
        d = dict()
        
        if cls._fieldnames:
            values = cls._struct.unpack(flo.read(cls._struct.size))
            d.update(dict(zip(cls._fieldnames,values)))
        
        _byte = cls._byte
        for key in cls._variable:
            _id_len, = _byte.unpack(flo.read(1))
            d[key] = str(flo.read(_id_len))
        
        return d
    
    @classmethod
    def unpack(cls,flo):
        return cls(**cls._unpack(flo))
    
    def pack(self):
        packed = ''
        
        if self._fieldnames:
            packed += self._struct.pack(\
                            *[getattr(self,fn) for fn in self._fieldnames])
            
        for key in self._variable:
            v = getattr(self,key)
            l = len(v)
            packed += struct.pack('B{n}s'.format(n = l),l,str(v))
        
        return packed
    
    @property
    def bytes(self):
        if not self._bytestring:
            self._bytestring = self.pack()
        return self._bytestring
    
    @property
    def keyvalue(self):
        return self._id,self.bytes

class BaseInsertion(BaseStruct):
    _variable = ('_id','content_type')

class Insertion(BaseInsertion):
    _fieldnames = ('offset','size','timestamp','digest')
    _struct = struct.Struct('3Q20s')
    
    @property
    def hexdigest(self):
        return hexlify(self.digest)

class Deletion(BaseStruct):
    _fieldnames = ('timestamp',)
    _struct = struct.Struct('Q')
