import unittest2
from uuid import uuid4
import os
import mmap

from file_like import ReadonlyFile

class MockInsertion(object):
    
    def __init__(self,offset,size):
        super(MockInsertion,self).__init__()
        self.offset = offset
        self.size = size
        self.timestamp = None
        self.hexdigest = None
        self.content_type = None

mi = MockInsertion

class ReadonlyFileTest(unittest2.TestCase):
    
    def setUp(self):
        self.data = 'Oliver is the cutest!'
        self.fn = '{x}.dat'.format(x = uuid4().hex)
        self.path = os.path.join('/tmp/',self.fn)
        with open(self.path,'wb') as f:
            f.write(self.data)
    
        self.f = open(self.path,'rb')
        self.map = mmap.mmap(self.f.fileno(),0,prot = mmap.PROT_READ)
     
    def tearDown(self):
        self.map.close()
        self.f.close()
        os.remove(self.path)
    
    @property
    def data_length(self):
        return len(self.data)
    
    def test_buffer_error_when_offset_and_length_are_out_of_bounds(self):
        self.assertRaises(\
            BufferError,lambda : ReadonlyFile(self.map,mi(10,self.data_length)))
    
    def test_negative_seek_relative_to_begin_raises_io_error(self):
        with ReadonlyFile(self.map,mi(10,2)) as rof:
            self.assertRaises(IOError,lambda : rof.seek(-1))
    
    def test_seek_past_start_of_file_relative_to_current_position_raises(self):
        with ReadonlyFile(self.map,mi(10,4)) as rof:
            rof.seek(2)
            self.assertRaises(IOError,lambda : rof.seek(-3,1))
    
    def test_seek_past_start_of_file_relative_to_end_raises(self):
        with ReadonlyFile(self.map,mi(10,4)) as rof:
            self.assertRaises(IOError,lambda : rof.seek(-5,2))
    
    def test_seek_past_end_of_file_tell_returns_specified_position(self):
        with ReadonlyFile(self.map,mi(10,4)) as rof:
            rof.seek(57)
            self.assertEqual(57,rof.tell())
    
    def test_seek_past_end_of_file_read_returns_empty_string(self):
        with ReadonlyFile(self.map,mi(10,4)) as rof:
            rof.seek(55)
            self.assertEqual('',rof.read())
    
    def test_read_with_no_arguments_returns_remaining_data(self):
        with ReadonlyFile(self.map,mi(10,4)) as rof:
            rof.seek(1)
            self.assertEqual('he ',rof.read())
    
    def test_read_changes_read_pointer_position(self):
        with ReadonlyFile(self.map,mi(10,4)) as rof:
            rof.read(2)
            self.assertEqual(2,rof.tell())
    
    def test_unknown_whence_argument_raises_io_error(self):
        with ReadonlyFile(self.map,mi(10,4)) as rof:
            self.assertRaises(IOError,lambda : rof.seek(1,3))
    
    def test_data_is_truncated_at_specified_bound(self):
        with ReadonlyFile(self.map,mi(1,4)) as rof:
            rof.seek(3)
            self.assertEqual('e',rof.read(1000))
            
    
    