import unittest2

class HashRing(object):
    
    def __init__(self,n_nodes,n_replicas,local_index):
        self.n_nodes = n_nodes
        self.n_replicas = n_replicas
        self.local_index = local_index
    
    def pos(self,point):
        return (self.local_index - point) % self.n_nodes
    
    def can_serve(self,point):
        return self.pos(point) < self.n_replicas

class HashRingTest(unittest2.TestCase):
    
    def test_can_serve_one_node(self):
        hr = HashRing(1,1,0)
        self.assertTrue(hr.can_serve(0))
        self.assertEqual(0,hr.pos(0))
    
    def test_can_serve_two_nodes_one_replica(self):
        hr = HashRing(2,1,0)
        self.assertTrue(hr.can_serve(0))
        self.assertEqual(0,hr.pos(0))
    
    def test_cannot_serve_two_nodes_one_replica(self):
        hr = HashRing(2,1,1)
        self.assertFalse(hr.can_serve(0))
    
    def test_can_serve_five_nodes_three_replicas(self):
        hr = HashRing(5,3,4)
        self.assertTrue(hr.can_serve(2))
        self.assertEqual(2,hr.pos(2))
    
    def test_cannot_serve_five_nodes_three_replicas(self):
        hr = HashRing(5,3,4)
        self.assertFalse(hr.can_serve(0))
    
    def test_can_serve_five_nodes_three_replicas_wraparound(self):
        hr = HashRing(5,3,1)
        self.assertTrue(hr.can_serve(4))
        self.assertEqual(2,hr.pos(4))
    
    def test_can_serve_five_nodes_three_replicas_wraparound_2(self):
        hr = HashRing(5,3,0)
        self.assertTrue(hr.can_serve(4))
        self.assertEqual(1,hr.pos(4))
    
    def test_can_serve_five_nodes_three_replicas_wraparound_3(self):
        hr = HashRing(5,3,0)
        self.assertTrue(hr.can_serve(3))
        self.assertEqual(2,hr.pos(3))
    
    def test_cannot_serve_five_nodes_three_replicas_wraparound(self):
        hr = HashRing(5,3,3)
        self.assertFalse(hr.can_serve(4))
    
    def test_a(self):
        hr = HashRing(3,2,0)
        self.assertTrue(hr.can_serve(0))
        self.assertEqual(0,hr.pos(0))
    
    def test_b(self):
        hr = HashRing(3,2,0)
        self.assertFalse(hr.can_serve(1))
    
    def test_c(self):
        hr = HashRing(3,2,0)
        self.assertTrue(hr.can_serve(2))
        self.assertEqual(1,hr.pos(2))
    
    def test_d(self):
        hr = HashRing(3,2,1)
        self.assertTrue(hr.can_serve(1))
        self.assertEqual(0,hr.pos(1))
    
    def test_e(self):
        hr = HashRing(3,2,1)
        self.assertFalse(hr.can_serve(2))
    
    def test_f(self):
        hr = HashRing(3,2,1)
        self.assertTrue(hr.can_serve(0))
        self.assertEqual(1,hr.pos(0))
    
    def test_g(self):
        hr = HashRing(3,2,2)
        self.assertTrue(hr.can_serve(2))
        self.assertEqual(0,hr.pos(2))
    
    def test_h(self):
        hr = HashRing(3,2,2)
        self.assertFalse(hr.can_serve(0))
    
    def test_i(self):
        hr = HashRing(3,2,2)
        self.assertTrue(hr.can_serve(1))
        self.assertEqual(1,hr.pos(1))
    
    
    
    